@@ -323,9 +323,8 @@ bail:
 
 }
 
-
 extern "C"
-int get_disc_size_cd (int fd)
+int get_disc_size_cdr (int fd)
 {
   Scsi_Command cmd(fd);
   int retval = -1;
@@ -366,6 +365,36 @@ bail:
   return retval;
 }
 
+extern "C"
+gint64 get_disc_size_cd (int fd)
+{
+  Scsi_Command cmd (fd);
+  unsigned char data [8];
+  unsigned int ccity, bsize;
+  int retval = -1;
+  int len;
+
+  /* For valgrind */
+  memset (&data, 0, sizeof (data));
+
+  cmd [0] = 0x25; /* READ CDROM CAPACITY */
+  cmd [9] = 0;
+
+  if (cmd.transport (READ, data, 8)) {    
+    goto bail;
+  }
+
+  ccity = (data [0] << 24) | (data [1] << 16) | (data [2] << 8) | data [3];
+  ccity++;
+  bsize = data [4] << 24 | data [5] << 16 | data [6] << 8 | data [7];
+
+  return ccity * bsize;
+
+bail:
+
+  return retval;
+}
+
 static unsigned int get_2k_capacity (Scsi_Command &cmd, int mmc_profile)
 { unsigned char	buf[32];
   unsigned int	ret=0;
