@@ -1,8 +1,9 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*-
  *
- * cd-drive.c: easy to use cd burner software
+ * nautilus-burn-drive.c: easy to use cd burner software
  *
  * Copyright (C) 2002-2004 Red Hat, Inc.
+ * Copyright (C) 2005 William Jon McCann <mccann@jhu.edu>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,15 +22,10 @@
  * 
  * Authors: Alexander Larsson <alexl@redhat.com>
  *          Bastien Nocera <hadess@hadess.net>
+ *          William Jon McCann <mccann@jhu.edu>
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
-
-#include <glib.h>
-#include <glib/gi18n.h>
-#include <gdk/gdk.h>
 
 #include <errno.h>
 #include <stdio.h>
@@ -42,14 +38,6 @@
 #include <fcntl.h>
 #include <math.h>
 
-#ifndef INVALID_HANDLE
-#define INVALID_HANDLE (GINT_TO_POINTER(-1))
-#endif
-
-#ifdef USE_HAL
-#include <libhal.h>
-#endif /* USE_HAL */
-
 #ifdef __linux__
 #include <linux/cdrom.h>
 #include <scsi/scsi.h>
@@ -62,16 +50,34 @@
 #include <camlib.h>
 #endif /* __FreeBSD__ */
 
+#ifdef HAVE_SYS_CDIO_H
+#include <sys/cdio.h>
+#endif
+
+#include <glib.h>
+#include <glib/gstdio.h>
+#include <glib/gi18n.h>
+#include <gdk/gdk.h>
+
+#ifdef USE_HAL
+#include <libhal.h>
+#endif /* USE_HAL */
+
 #include "nautilus-burn-drive.h"
 
 /* For dvd_plus_rw_utils.cpp */
 int    get_dvd_r_rw_profile (const char *name);
 int    get_mmc_profile      (int fd);
-int    get_disc_size_cd     (int fd);
+gint64 get_disc_size_cd     (int fd);
+int    get_disc_size_cdr    (int fd);
 gint64 get_disc_size_dvd    (int fd, int mmc_profile);
 int    get_read_write_speed (int fd, int *read_speed, int *write_speed);
 int    get_disc_status      (int fd, int *empty, int *is_rewritable, int *is_blank);
 
+#ifndef INVALID_HANDLE
+#define INVALID_HANDLE (GINT_TO_POINTER(-1))
+#endif
+
 #define CD_ROM_SPEED 176
 
 static struct {
@@ -124,8 +130,8 @@ open_ioctl_handle (const char *device)
 {
 	int fd;
 
- 	if ((fd = open (device, O_RDWR | O_EXCL | O_NONBLOCK)) < 0
- 	    && (fd = open (device, O_RDONLY | O_EXCL | O_NONBLOCK)) < 0) {
+ 	if ((fd = g_open (device, O_RDWR | O_EXCL | O_NONBLOCK, 0)) < 0
+ 	    && (fd = g_open (device, O_RDONLY | O_EXCL | O_NONBLOCK, 0)) < 0) {
 		return INVALID_HANDLE;
 	}
 
@@ -145,18 +151,38 @@ static LibHalContext *
 get_hal_context (void)
 {
 	static LibHalContext *ctx = NULL;
-	LibHalFunctions       hal_functions = {
-		NULL, /* mainloop integration */
-		NULL, /* device_added */
-		NULL, /* device_removed */
-		NULL, /* device_new_capability */
-		NULL, /* device_lost_capability */
-		NULL, /* property_modified */
-		NULL, /* device_condition */
-	};
+	DBusError             error;
+	DBusConnection       *dbus_conn;
 
-	if (ctx == NULL)
-		ctx = hal_initialize (&hal_functions, FALSE);
+	if (ctx == NULL) {
+		ctx = libhal_ctx_new ();
+		if (ctx == NULL) {
+			g_warning ("Could not create a HAL context\n");
+		} else { 
+			dbus_error_init (&error);
+			dbus_conn = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
+
+			if (dbus_error_is_set (&error)) {
+				g_warning ("Could not connect to system bus: %s\n", error.message);
+				dbus_error_free (&error);
+				return NULL;
+			}
+
+			libhal_ctx_set_dbus_connection (ctx, dbus_conn);
+
+			if (! libhal_ctx_init (ctx, &error)) {
+				g_warning ("Could not initalize "
+					   "the HAL context: %s\n",
+					   error.message);
+
+				if (dbus_error_is_set (&error))
+					dbus_error_free (&error);
+
+				libhal_ctx_free (ctx);
+				ctx = NULL;
+			}
+		}
+	}
 
 	return ctx;
 }
@@ -180,7 +206,7 @@ get_device_max_write_speed (char *device)
 	fd = get_ioctl_handle_fd (ioctl_handle);
 	get_read_write_speed (fd, &read_speed, &write_speed);
 	close_ioctl_handle (ioctl_handle);
-	max_speed = (int)floor  (write_speed) / CD_ROM_SPEED;
+	max_speed = (int) floor (write_speed) / CD_ROM_SPEED;
 
 	return max_speed;
 }
@@ -340,9 +366,12 @@ add_dvd_plus (NautilusBurnDrive *drive)
 }
 
 static gboolean
-nautilus_burn_drive_door_open (gboolean mmc_profile,
-			       int      fd)
+drive_door_is_open (int fd)
 {
+	if (fd < 0) {
+		return FALSE;
+	}
+
 #ifdef __linux__
 	{
 		int status;
@@ -355,14 +384,185 @@ nautilus_burn_drive_door_open (gboolean mmc_profile,
 		return status == CDS_TRAY_OPEN;
 	}
 #else
-	if (mmc_profile == 0) {
-		return TRUE;
-	}
-
 	return FALSE;
 #endif
 }
 
+gboolean
+nautilus_burn_drive_door_is_open (NautilusBurnDrive *drive)
+{
+	gpointer ioctl_handle;
+	int      fd;
+	gboolean ret;
+
+	g_return_val_if_fail (drive != NULL, FALSE);
+
+	ioctl_handle = open_ioctl_handle (drive->device);
+	if (ioctl_handle == INVALID_HANDLE)
+		return FALSE;
+
+	fd = get_ioctl_handle_fd (ioctl_handle);
+
+	ret = drive_door_is_open (fd);
+
+	close_ioctl_handle (ioctl_handle);
+
+	return ret;
+}
+
+/* adapted from gnome-vfs */
+static int
+get_disc_type (const char *dev_path)
+{
+	int                   fd = 0;
+#if defined (HAVE_SYS_MNTTAB_H)   /* Solaris */
+	GString              *new_dev_path;
+	struct cdrom_tocentry entry;
+	struct cdrom_tochdr   header;
+	int                   type;
+
+	/* For ioctl call to work dev_path has to be /vol/dev/rdsk.
+	 * There has to be a nicer way to do this.
+	 */
+	new_dev_path = g_string_new (dev_path);
+	new_dev_path = g_string_insert_c (new_dev_path, 9, 'r');
+	fd = open (new_dev_path->str, O_RDONLY | O_NONBLOCK);
+	g_string_free (new_dev_path, TRUE);
+
+	if (fd < 0) {
+		return -1;
+	}
+
+	if (ioctl (fd, CDROMREADTOCHDR, &header) == 0) {
+		return -1;
+	}
+
+	type = CDS_DATA_1;
+	
+	for (entry.cdte_track = 1;
+	     entry.cdte_track <= header.cdth_trk1;
+	     entry.cdte_track++) {
+		entry.cdte_format = CDROM_LBA;
+		if (ioctl (fd, CDROMREADTOCENTRY, &entry) == 0) {
+			if (entry.cdte_ctrl & CDROM_DATA_TRACK) {
+				type = CDS_AUDIO;
+				break;
+			}
+		}
+	}
+
+	return type;
+#elif defined(HAVE_SYS_MNTCTL_H)
+	return CDS_NO_INFO;
+#elif defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__DragonFly__)
+	struct ioc_toc_header            header;
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || defined(__DragonFly__)
+	struct ioc_read_toc_single_entry entry;
+#else
+	struct ioc_read_toc_entry        entries;
+	struct cd_toc_entry              entry;
+	int                              i;
+#endif
+	int                              type;
+#ifndef CDROM_DATA_TRACK
+#define CDROM_DATA_TRACK 4
+#endif
+
+	fd = open (dev_path, O_RDONLY|O_NONBLOCK);
+	if (fd < 0) {
+	    	return -1;
+	}
+
+	if (ioctl (fd, CDIOREADTOCHEADER, &header) == 0) {
+	    	return -1;
+	}
+
+	type = CDS_DATA_1;
+#if defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || defined(__DragonFly__)
+	for (entry.track = header.starting_track;
+		entry.track <= header.ending_track;
+		entry.track++) {
+	    	entry.address_format = CD_LBA_FORMAT;
+		if (ioctl (fd, CDIOREADTOCENTRY, &entry) == 0) {
+		    	if (entry.entry.control & CDROM_DATA_TRACK) {
+			    	type = CDS_AUDIO;
+				break;
+			}
+		}
+	}
+
+#else /* defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || defined(__DragonFly__)*/
+	entries.data_len = sizeof (entry);
+	entries.data = &entry;
+	for (i = header.starting_track; i <= header.ending_track; i++) {
+		entries.starting_track = i;
+	    	entries.address_format = CD_LBA_FORMAT;
+		if (ioctl (fd, CDIOREADTOCENTRYS, &entries) == 0) {
+		    	if (entry.control & CDROM_DATA_TRACK) {
+			    	type = CDS_AUDIO;
+				break;
+			}
+		}
+	}
+
+#endif /* defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || defined(__DragonFly__)*/
+	return type;
+#elif defined(__linux__)
+	fd = open (dev_path, O_RDONLY|O_NONBLOCK);
+	if (fd  < 0) {
+		return -1;
+	}
+	if (ioctl (fd, CDROM_DRIVE_STATUS, CDSL_CURRENT) != CDS_DISC_OK) {
+		close (fd);
+		fd = -1;
+		return -1;
+	}
+	return ioctl (fd, CDROM_DISC_STATUS, CDSL_CURRENT);
+#else /* defined(__linux__) */
+	return CDS_NO_INFO;
+#endif /* defined(__linux__) */
+}
+
+static void
+disc_type_get_info (int       type,
+		    gboolean *is_blank,
+		    gboolean *has_data,
+		    gboolean *has_audio)
+{
+	gboolean _has_audio = FALSE;
+	gboolean _has_data  = FALSE;
+	gboolean _is_blank  = FALSE;
+
+	switch (type) {
+	case CDS_AUDIO:		/* audio CD */
+		_has_audio = TRUE;
+		break;
+	case CDS_MIXED:		/* mixed mode CD */
+		_has_audio = TRUE;
+		_has_data  = TRUE;
+		break;
+	case CDS_DATA_1:	/* data CD */
+	case CDS_DATA_2:
+	case CDS_XA_2_1:
+	case CDS_XA_2_2:
+		_has_data  = TRUE;
+		break;
+	case CDS_NO_INFO:	/* blank or invalid CD */
+		_is_blank  = TRUE;
+		break;
+		
+	default:		/* should never see this */
+		break;
+	}
+
+	if (is_blank)
+		*is_blank = _is_blank;
+	if (has_data)
+		*has_data = _has_data;
+	if (has_audio)
+		*has_audio = _has_audio;
+}
+	
 static NautilusBurnMediaType
 nautilus_burn_drive_get_media_type_from_path_full (const char *device,
 						   gboolean   *is_rewritable,
@@ -373,6 +573,7 @@ nautilus_burn_drive_get_media_type_from_path_full (const char *device,
 	gpointer ioctl_handle;
 	int      fd;
 	int      mmc_profile;
+	int      disc_type;
 
 	g_return_val_if_fail (device != NULL, NAUTILUS_BURN_MEDIA_TYPE_ERROR);
 
@@ -386,6 +587,7 @@ nautilus_burn_drive_get_media_type_from_path_full (const char *device,
 		if (errno == EBUSY) {
 			return NAUTILUS_BURN_MEDIA_TYPE_BUSY;
 		}
+		g_warning ("Could not open %s: %s", device, g_strerror (errno));
 		return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
 	}
 
@@ -394,15 +596,16 @@ nautilus_burn_drive_get_media_type_from_path_full (const char *device,
 
 	/* Couldn't get the data about the media */
 	if (mmc_profile < 0) {
-		gboolean opened;
+		gboolean door_is_open;
 
-		opened = nautilus_burn_drive_door_open (mmc_profile, fd);
+		door_is_open = drive_door_is_open (fd);
 
-		if (opened != FALSE) {
+		if (door_is_open != FALSE) {
 			close_ioctl_handle (ioctl_handle);
 			return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
 		} else {
 			int blank, rewrite, empty;
+
 			if (get_disc_status (fd, &empty, &rewrite, &blank) == 0) {
 				close_ioctl_handle (ioctl_handle);
 
@@ -410,12 +613,10 @@ nautilus_burn_drive_get_media_type_from_path_full (const char *device,
 					*is_rewritable = rewrite;
 				if (is_blank)
 					*is_blank = blank;
-				if (has_data)
-					*has_data = !blank;
-				if (has_audio) {
-					/* FIXME */
-					*has_audio = FALSE;
-				}
+
+				disc_type = get_disc_type (device);
+				disc_type_get_info (disc_type, is_blank, has_data, has_audio);
+
 				if (empty)
 					return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
 				else
@@ -426,18 +627,17 @@ nautilus_burn_drive_get_media_type_from_path_full (const char *device,
 
 			return NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
 		}
+	} else {
+		g_warning ("Could not read MMC profile of %s", device);
 	}
 
 	close_ioctl_handle (ioctl_handle);
 
 	if (is_blank)
 		*is_blank = mmc_profile & 0x10000;
-	if (has_data)
-		*has_data = !(mmc_profile & 0x10000);
-	if (has_audio) {
-		/* FIXME */
-		*has_audio = FALSE;
-	}
+
+	disc_type = get_disc_type (device);
+	disc_type_get_info (disc_type, is_blank, has_data, has_audio);
 
 	switch (mmc_profile & 0xFFFF) {
 	case -1:
@@ -463,6 +663,8 @@ nautilus_burn_drive_get_media_type_from_path_full (const char *device,
 		return NAUTILUS_BURN_MEDIA_TYPE_DVDRW;
 	case 0x1B:      /* DVD+R                        */
 		return NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R;
+	case 0x2B:      /* DVD+R Double Layer           */
+		return NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R_DL;
 	case 0x1A:      /* DVD+RW                       */
 		if (is_rewritable)
 			*is_rewritable = TRUE;
@@ -530,43 +732,67 @@ nautilus_burn_drive_hal_get_media_type_full (NautilusBurnDrive *drive,
 		int                   num_devices;
 		NautilusBurnMediaType type;
 		char                 *hal_type;
+		DBusError             error;
 		
 		ctx = get_hal_context ();
+
+		dbus_error_init (&error);
 		if (ctx != NULL) {
-			device_names = hal_manager_find_device_string_match (ctx, 
-									     "info.parent",
-									     drive->priv->udi,
-									     &num_devices);
+			device_names = libhal_manager_find_device_string_match (ctx, 
+										"info.parent",
+										drive->priv->udi,
+										&num_devices,
+										&error);
+
+			if (dbus_error_is_set (&error)) {
+				g_warning ("%s\n", error.message);
+				dbus_error_free (&error);
+				return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
+			}
+
 			if (num_devices == 0) {
+				g_warning ("No HAL devices found for UDI '%s'", drive->priv->udi);
 				return NAUTILUS_BURN_MEDIA_TYPE_ERROR;
 			}
 
 			/* just look at the first child */
-			if (hal_device_get_property_bool (ctx, 
-							  device_names [0],
-							  "volume.is_mounted")) {
+			if (libhal_device_get_property_bool (ctx, 
+							     device_names [0],
+							     "volume.is_mounted",
+							     NULL)) {
 				type = NAUTILUS_BURN_MEDIA_TYPE_BUSY;
-			} else {		    
+			} else {
+			
 				if (is_rewritable)
-					*is_rewritable = hal_device_get_property_bool (ctx, 
-										       device_names [0],
-										       "volume.disc.is_rewritable");
+					*is_rewritable = libhal_device_get_property_bool (ctx, 
+									 		  device_names [0],
+											  "volume.disc.is_rewritable",
+											  NULL);
+
 				if (is_blank)
-					*is_blank = hal_device_get_property_bool (ctx, 
-										  device_names [0],
-										  "volume.disc.is_blank");
+					*is_blank = libhal_device_get_property_bool (ctx, 
+										     device_names [0],
+										     "volume.disc.is_blank",
+										     NULL);
+
 				if (has_data)
-					*has_data = hal_device_get_property_bool (ctx, 
-										  device_names [0],
-										  "volume.disc.has_data");
+					*has_data = libhal_device_get_property_bool (ctx, 
+										     device_names [0],
+										     "volume.disc.has_data",
+										     NULL);
+
 				if (has_audio)
-					*has_audio = hal_device_get_property_bool (ctx, 
-										  device_names [0],
-										  "volume.disc.has_audio");
+					*has_audio = libhal_device_get_property_bool (ctx, 
+										      device_names [0],
+										      "volume.disc.has_audio",
+										      NULL);
+
 				type = NAUTILUS_BURN_MEDIA_TYPE_BUSY;
-				hal_type = hal_device_get_property_string (ctx, 
-									   device_names [0],
-									   "volume.disc.type");
+				hal_type = libhal_device_get_property_string (ctx, 
+									      device_names [0],
+									      "volume.disc.type",
+									      NULL);
+
 				if (hal_type == NULL || strcmp (hal_type, "unknown") == 0) {
 					type = NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
 				} else if (strcmp (hal_type, "cd_rom") == 0) {
@@ -587,15 +813,17 @@ nautilus_burn_drive_hal_get_media_type_full (NautilusBurnDrive *drive,
 					type = NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW;
 				} else if (strcmp (hal_type, "dvd_plus_r") == 0) {
 					type = NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R;
+				} else if (strcmp (hal_type, "dvd_plus_r_dl") == 0) {
+					type = NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R_DL;
 				} else {
 					type = NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
 				}
 				
 				if (hal_type != NULL)
-					hal_free_string (hal_type);
+					libhal_free_string (hal_type);
 			}
 
-			hal_free_string_array (device_names);
+			libhal_free_string_array (device_names);
 
 			return type;
 		}
@@ -646,23 +874,13 @@ nautilus_burn_drive_get_media_type_full (NautilusBurnDrive *drive,
 }
 
 /**
- * nautilus_burn_drive_get_media_type_and_rewritable:
- * @drive: #NautilusBurnDrive
- * @is_rewritable: set to TRUE if drive is rewritable
+ * nautilus_burn_drive_get_media_size_from_path:
+ * @device: Filename of drive device (eg. "/dev/hdb")
  *
- * Get the type of the specified drive.
+ * Determine the capacity of the media in the specified device.
  *
- * Return value: See nautilus_burn_drive_get_media_type_from_path() for details.
+ * Return value: The capacity of the media in bytes.
  **/
-NautilusBurnMediaType
-nautilus_burn_drive_get_media_type_and_rewritable (NautilusBurnDrive *drive,
-						   gboolean          *is_rewritable)
-{
-	g_return_val_if_fail (drive != NULL, NAUTILUS_BURN_MEDIA_TYPE_ERROR);
-
-	return nautilus_burn_drive_get_media_type_full (drive, is_rewritable, NULL, NULL, NULL);
-}
-
 gint64
 nautilus_burn_drive_get_media_size_from_path (const char *device)
 {
@@ -689,15 +907,22 @@ nautilus_burn_drive_get_media_size_from_path (const char *device)
 
 	/* See nautilus_burn_drive_get_media_type_from_path for details */
 	switch (mmc_profile & 0xFFFF) {
+	case 0x8:
+		size = get_disc_size_cd (fd);
+		break;
 	case 0x9:
 	case 0xa:
-		secs = get_disc_size_cd (fd);
-		size = (1 + secs * 7 / 48) * 1024 * 1024;
+		secs = get_disc_size_cdr (fd);
+		if (secs > 0)
+			size = (1 + secs * 7 / 48) * 1024 * 1024;
+		else
+			size = 0;
 		break;
 	case 0x11:
 	case 0x13:
 	case 0x14:
 	case 0x1B:
+	case 0x2B:
 	case 0x1A:
 	case 0x12:
 		size = get_disc_size_dvd (fd, mmc_profile);
@@ -717,7 +942,7 @@ nautilus_burn_drive_get_media_size_from_path (const char *device)
  *
  * Determine the capacity of the media in the specified drive.
  *
- * Return value: See #nautilus_burn_drive_get_media_size_from_path for details.
+ * Return value: The capacity of the media in bytes.
  **/
 gint64
 nautilus_burn_drive_get_media_size (NautilusBurnDrive *drive)
@@ -858,11 +1083,23 @@ unmount_thread_start (void *arg)
 	return NULL;
 }
 
+/**
+ * nautilus_burn_drive_unmount:
+ * @drive: #NautilusBurnDrive
+ *
+ * Unmount the media in a #NautilusBurnDrive.
+ *
+ * Return value: %TRUE if the media was sucessfully unmounted, %FALSE otherwise.
+ *
+ * Since: 2.10
+ **/
 gboolean
 nautilus_burn_drive_unmount (NautilusBurnDrive *drive)
 {
 	UnmountData *data;
 	gboolean     unmount_ok;
+
+	g_return_val_if_fail (drive != NULL, FALSE);
 	
 	if (drive->device == NULL)
 		return FALSE;
@@ -872,7 +1109,7 @@ nautilus_burn_drive_unmount (NautilusBurnDrive *drive)
 	data = g_new0 (UnmountData, 1);
 	data->loop = g_main_loop_new (NULL, FALSE);
 
-	data->timeout_tag = g_timeout_add (5*1000,
+	data->timeout_tag = g_timeout_add (5 * 1000,
 					   unmount_timeout,
 					   data);
 	data->command = find_command (umount_known_locations);
@@ -896,9 +1133,40 @@ nautilus_burn_drive_unmount (NautilusBurnDrive *drive)
 	return unmount_ok;
 }
 
+/**
+ * nautilus_burn_drive_eject:
+ * @drive: #NautilusBurnDrive
+ *
+ * Eject media from a #NautilusBurnDrive.
+ *
+ * Return value: %TRUE if the media was sucessfully ejected, %FALSE otherwise.
+ *
+ * Since: 2.12
+ **/
+gboolean
+nautilus_burn_drive_eject (NautilusBurnDrive *drive)
+{
+	char    *cmd;
+	gboolean res;
+
+	g_return_val_if_fail (drive != NULL, FALSE);
+
+	if (drive->device == NULL)
+		return FALSE;
+	
+	cmd = g_strdup_printf ("eject %s", drive->device);
+	res = g_spawn_command_line_sync (cmd, NULL, NULL, NULL, NULL);
+	g_free (cmd);
+
+	/* delay a bit to make sure eject finishes */
+	sleep (2);
+
+	return res;
+}
+
 #ifdef USE_HAL
 
-#define GET_BOOL_PROP(x) (hal_device_property_exists (ctx, device_names [i], x) && hal_device_get_property_bool (ctx, device_names [i], x))
+#define GET_BOOL_PROP(x) (libhal_device_property_exists (ctx, device_names [i], x, NULL) && libhal_device_get_property_bool (ctx, device_names [i], x, NULL))
 
 static GList *
 hal_scan (gboolean recorder_only)
@@ -914,8 +1182,10 @@ hal_scan (gboolean recorder_only)
 		return NULL;
 	}
 
-	device_names = hal_find_device_by_capability (ctx,
-						      "storage.cdrom", &num_devices);
+	device_names = libhal_find_device_by_capability (ctx,
+						         "storage.cdrom", 
+							 &num_devices,
+							 NULL);
 
 	if (device_names == NULL)
 		return NULL;
@@ -930,53 +1200,69 @@ hal_scan (gboolean recorder_only)
 
 		drive = nautilus_burn_drive_new ();
 		drive->type = NAUTILUS_BURN_DRIVE_TYPE_CD_DRIVE;
+
 		if (is_cdr != FALSE) {
 			drive->type |= NAUTILUS_BURN_DRIVE_TYPE_CD_RECORDER;
 		}
+
 		if (GET_BOOL_PROP ("storage.cdrom.cdrw")) {
 			drive->type |= NAUTILUS_BURN_DRIVE_TYPE_CDRW_RECORDER;
 		}
+
 		if (GET_BOOL_PROP ("storage.cdrom.dvd")) {
 			drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_DRIVE;
 
 			if (GET_BOOL_PROP ("storage.cdrom.dvdram")) {
 				drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_RAM_RECORDER;
 			}
+
 			if (GET_BOOL_PROP ("storage.cdrom.dvdr")) {
 				drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_RW_RECORDER;
 			}
+
 			if (GET_BOOL_PROP ("storage.cdrom.dvd")) {
 				drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_DRIVE;
 			}
+
 			if (GET_BOOL_PROP ("storage.cdrom.dvdplusr")) {
 				drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_R_RECORDER;
 			}
+
 			if (GET_BOOL_PROP ("storage.cdrom.dvdplusrw")) {
 				drive->type |= NAUTILUS_BURN_DRIVE_TYPE_DVD_PLUS_RW_RECORDER;
 			}
+
 		}
 
-		drive->device = hal_device_get_property_string (ctx,
-								device_names [i], "block.device");
+		drive->device = libhal_device_get_property_string (ctx,
+								   device_names [i], 
+								   "block.device",
+								   NULL);
+
 		drive->cdrecord_id = g_strdup (drive->device);
 
-		string = hal_device_get_property_string (ctx,
-							 device_names [i], "storage.model");
+		string = libhal_device_get_property_string (ctx,
+							    device_names [i], 
+							    "storage.model",
+							    NULL);
+
 		if (string != NULL) {
 			drive->display_name = string;
 		} else {
 			drive->display_name = g_strdup_printf ("Unnamed Drive (%s)", drive->device);
 		}
 
-		drive->max_speed_read = hal_device_get_property_int
-			(ctx, device_names [i], "storage.cdrom.read_speed")
+		drive->max_speed_read = libhal_device_get_property_int
+			(ctx, device_names [i], "storage.cdrom.read_speed", NULL)
 			/ CD_ROM_SPEED;
 
-		if (hal_device_property_exists (ctx, device_names [i], "storage.cdrom.write_speed")) {
-			drive->max_speed_write = hal_device_get_property_int
+		if (libhal_device_property_exists (ctx, device_names [i], "storage.cdrom.write_speed", NULL)) {
+			drive->max_speed_write = libhal_device_get_property_int
 				(ctx, device_names [i],
-				 "storage.cdrom.write_speed")
+				 "storage.cdrom.write_speed",
+				 NULL)
 				/ CD_ROM_SPEED;
+
 		}
 
 		add_whitelist (drive);
@@ -990,7 +1276,7 @@ hal_scan (gboolean recorder_only)
 		drive->priv->udi = g_strdup (device_names [i]);
 	}
 
-	hal_free_string_array (device_names);
+	libhal_free_string_array (device_names);
 
 	drives = g_list_reverse (drives);
 
@@ -1158,9 +1444,9 @@ get_cd_scsi_id (const char *dev,
 	} m_idlun;
 	
 	devfile = g_strdup_printf ("/dev/%s", dev);
-	fd = open (devfile, O_RDWR | O_NONBLOCK);
+	fd = g_open (devfile, O_RDWR | O_NONBLOCK, 0);
 	if (fd < 0)
-		fd = open (devfile, O_RDONLY | O_NONBLOCK);
+		fd = g_open (devfile, O_RDONLY | O_NONBLOCK, 0);
 
 	g_free (devfile);
 
@@ -1242,14 +1528,14 @@ drive_get_name (struct drive_unit *drive,
 	g_return_val_if_fail (drive != NULL, FALSE);
 
 	/* clean up the string again if we have devfs */
-	i = sscanf(drive->device, "%4s %14s", stdname, devfsname);
+	i = sscanf (drive->device, "%4s %14s", stdname, devfsname);
 	if (i < 1) { /* should never happen */
-		g_warning("drive_get_name: drive->device string broken!");
+		g_warning ("drive_get_name: drive->device string broken!");
 		return NULL;
 	}
 	if (i == 2) {
 		g_free (drive->device);
-		drive->device = g_strdup(devfsname);
+		drive->device = g_strdup (devfsname);
 	}
 	stdname [3] = '\0'; devfsname [14] = '\0'; /* just in case */
 	
@@ -1450,7 +1736,7 @@ linux_scan (gboolean recorder_only)
 
 	if (n_scsi_units > 0) {
 		/* open /dev/sg0 to force loading of the sg module if not loaded yet */
-		fd = open ("/dev/sg0", O_RDWR);
+		fd = g_open ("/dev/sg0", O_RDWR, 0);
 		if (fd >= 0) {
 			close (fd);
 		}
@@ -1613,7 +1899,7 @@ freebsd_scan (gboolean recorder_only)
 
 	while (1) {
 		NautilusBurnDrive *drive;
-		gchar             *cam_path;
+		char              *cam_path;
 		struct cam_device *cam_dev;
 
 		cam_path = g_strdup_printf ("/dev/%s%dc", dev_type, i);
@@ -1906,14 +2192,21 @@ nautilus_burn_drive_lock (NautilusBurnDrive *drive,
 #ifdef USE_HAL
 	if (drive->priv->udi != NULL) {
 		LibHalContext *ctx;
-		char *dbus_reason;
+		char          *dbus_reason;
+		DBusError      error;
 		
+		dbus_error_init (&error);
 		ctx = get_hal_context ();
 		if (ctx != NULL) {
-			res = hal_device_lock (ctx, 
+			res = libhal_device_lock (ctx, 
 					       drive->priv->udi,
 					       reason,
-					       &dbus_reason);
+					       &dbus_reason,
+					       &error);
+
+			if (dbus_error_is_set (&error))
+				dbus_error_free (&error);
+				
 			if (dbus_reason != NULL && 
 			    reason_for_failure != NULL)
 				*reason_for_failure = g_strdup (dbus_reason);
@@ -1944,11 +2237,17 @@ nautilus_burn_drive_unlock (NautilusBurnDrive *drive)
 #ifdef USE_HAL
 	if (drive->priv->udi != NULL) {
 		LibHalContext *ctx;
+		DBusError      error;
 
+		dbus_error_init (&error);
 		ctx = get_hal_context ();
 		if (ctx != NULL) {
-			res = hal_device_unlock (ctx, 
-						 drive->priv->udi);
+			res = libhal_device_unlock (ctx, 
+						    drive->priv->udi,
+						    &error);
+
+			if (dbus_error_is_set (&error))
+				dbus_error_free (&error);
 		}
 	}
 #endif
@@ -1974,6 +2273,39 @@ nautilus_burn_drive_new (void)
 	return drive;
 }
 
+/**
+ * nautilus_burn_drive_new_from_path:
+ * @device: Filename of drive device (eg. "/dev/hdb")
+ *
+ * Create a new #NautilusBurnDrive.
+ *
+ * Return value: The new drive.
+ *
+ * Since: 2.12
+ **/
+NautilusBurnDrive *
+nautilus_burn_drive_new_from_path (const char *device)
+{
+	GList             *drives, *l;
+	NautilusBurnDrive *drive;
+
+        drives = nautilus_burn_drive_get_list (FALSE, FALSE);
+
+	drive = NULL;
+
+        for (l = drives; l != NULL; l = l->next) {
+		NautilusBurnDrive *d = l->data;
+		if (g_str_equal (device, d->device)) {
+			drive = nautilus_burn_drive_copy (d);
+		}
+	}
+
+	g_list_foreach (drives, (GFunc)nautilus_burn_drive_free, NULL);
+	g_list_free (drives);
+
+	return drive;
+}
+
 /**
  * nautilus_burn_drive_copy:
  * @drive: Pointer to the #NautilusBurnDrive to be copied
@@ -2028,3 +2360,50 @@ nautilus_burn_drive_equal (NautilusBurnDrive *a,
 
 	return strcmp (a->device, b->device) == 0;
 }
+
+/**
+ * nautilus_burn_drive_media_type_get_string:
+ * @type: A #NautilusBurnMediaType
+ *
+ * Get a string description of the specified media type.
+ *
+ * Returns: a string description for the media type.
+ *
+ * Since: 2.12
+ **/
+const char *
+nautilus_burn_drive_media_type_get_string (NautilusBurnMediaType type)
+{
+	switch (type) {
+	case NAUTILUS_BURN_MEDIA_TYPE_BUSY:
+		return _("Unknown media, CD drive is busy");
+	case NAUTILUS_BURN_MEDIA_TYPE_ERROR:
+		return _("Couldn't open media");
+	case NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN:
+		return _("Unknown Media");
+	case NAUTILUS_BURN_MEDIA_TYPE_CD:
+		return _("Commercial CD or Audio CD");
+	case NAUTILUS_BURN_MEDIA_TYPE_CDR:
+		return _("CD-R");
+	case NAUTILUS_BURN_MEDIA_TYPE_CDRW:
+		return _("CD-RW");
+	case NAUTILUS_BURN_MEDIA_TYPE_DVD:
+		return _("DVD");
+	case NAUTILUS_BURN_MEDIA_TYPE_DVDR:
+		return _("DVD-R, or DVD-RAM");
+	case NAUTILUS_BURN_MEDIA_TYPE_DVDRW:
+		return _("DVD-RW");
+	case NAUTILUS_BURN_MEDIA_TYPE_DVD_RAM:
+		return _("DVD-RAM");
+	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R:
+		return _("DVD+R");
+	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_RW:
+		return _("DVD+RW");
+	case NAUTILUS_BURN_MEDIA_TYPE_DVD_PLUS_R_DL:
+		return _("DVD+R DL");
+	default:
+		break;
+	}
+
+	return _("Broken media type");
+}
